@@ -47,7 +47,7 @@ class Catcher(object):
 
 def run(command_line, manager_run):
     '''
-        Returns tuple of standard output and exit code
+        Runs a manager command line, returns exit code
     '''
     sys.argv = command_line.split()
     exit_code = None
@@ -66,6 +66,31 @@ class SimpleCommand(Command):
         print('OK')
 
 
+class NamedCommand(Command):
+    'named command'
+
+    def run(self):
+        print('OK')
+
+
+class ExplicitNamedCommand(Command):
+    'named command'
+
+    name = 'named'
+
+    def run(self):
+        print('OK')
+
+
+class NamespacedCommand(Command):
+    'namespaced command'
+
+    namespace = 'ns'
+
+    def run(self):
+        print('OK')
+
+
 class CommandWithArgs(Command):
     'command with args'
 
@@ -122,24 +147,28 @@ class CommandWithCatchAll(Command):
         print(remaining_args)
 
 
-class TestCommands(unittest.TestCase):
+class EmptyContext(object):
+    def __enter__(self):
+        pass
+    def __exit__(self, a,b,c):
+        pass
 
-    TESTING = True
-
-    def setup(self):
-
-        self.app = Flask(__name__)
-        self.app.config.from_object(self)
+class TestApp(object):
+    def __init__(self, verbose=False):
+        self.verbose = verbose
+    def test_request_context(self):
+        return EmptyContext()
+    def __call__(self,**kw):
+        if self.verbose:
+            print("APP "+" ".join("%s=%s" % (k,v) for k,v in kw.items()))
+        return self
 
 
 class TestManager:
 
-    TESTING = True
-
     def setup(self):
 
-        self.app = Flask(__name__)
-        self.app.config.from_object(self)
+        self.app = TestApp()
 
     def test_with_default_commands(self):
 
@@ -162,6 +191,54 @@ class TestManager:
 
         assert isinstance(manager._commands['simple'], SimpleCommand)
 
+    def test_add_named_command(self):
+
+        manager = Manager(self.app)
+        manager.add_command(NamedCommand())
+
+        assert 'named' in manager._commands
+        assert isinstance(manager._commands['named'], NamedCommand)
+
+    def test_add_explicit_named_command(self):
+
+        manager = Manager(self.app)
+        manager.add_command(ExplicitNamedCommand())
+
+        name = ExplicitNamedCommand.name
+        assert name in manager._commands
+        assert isinstance(manager._commands[name], ExplicitNamedCommand)
+
+    def test_add_namespaced_command(self):
+
+        manager = Manager(self.app)
+        manager.add_command('one', NamespacedCommand())
+        manager.add_command('two', NamespacedCommand())
+
+        assert 'ns' in manager._commands
+        assert isinstance(manager._commands['ns'], Manager)
+        ns = manager._commands['ns']
+        assert isinstance(ns._commands['one'], NamespacedCommand)
+        assert isinstance(ns._commands['two'], NamespacedCommand)
+
+    def test_add_namespaced_simple_command(self):
+
+        manager = Manager(self.app)
+        manager.add_command('hello', SimpleCommand(), namespace='ns')
+        manager.add_command('world', SimpleCommand(), namespace='ns')
+
+        assert 'ns' in manager._commands
+        assert isinstance(manager._commands['ns'], Manager)
+        ns = manager._commands['ns']
+        assert isinstance(ns._commands['hello'], SimpleCommand)
+        assert isinstance(ns._commands['world'], SimpleCommand)
+
+    def test_add_command_class(self):
+
+        manager = Manager(self.app)
+        manager.add_command('simple', SimpleCommand)
+
+        assert isinstance(manager._commands['simple'], SimpleCommand)
+
     def test_simple_command_decorator(self, capsys):
 
         manager = Manager(self.app)
@@ -172,7 +249,7 @@ class TestManager:
 
         assert 'hello' in manager._commands
 
-        code = run('manage.py hello', lambda: manager.run())
+        code = run('manage.py hello', manager.run)
         out, err = capsys.readouterr()
         assert 'hello' in out
 
@@ -186,6 +263,22 @@ class TestManager:
 
         assert 'hello' in manager._commands
 
+        code = run('manage.py hello joe', manager.run)
+        out, err = capsys.readouterr()
+        assert 'hello joe' in out
+
+    def test_method_command_decorator_with_pos_arg(self, capsys):
+
+        manager = Manager(self.app)
+
+        class SomeTest(object):
+            def hello(self,name):
+                print('hello ' + name)
+        sometest = SomeTest()
+        manager.command(sometest.hello)
+
+        assert 'hello' in manager._commands
+
         code = run('manage.py hello joe', lambda: manager.run())
         out, err = capsys.readouterr()
         assert 'hello joe' in out
@@ -201,19 +294,19 @@ class TestManager:
 
         assert 'hello' in manager._commands
 
-        code = run('manage.py hello --name=joe', lambda: manager.run())
+        code = run('manage.py hello --name=joe', manager.run)
         out, err = capsys.readouterr()
         assert 'hello joe' in out
 
-        code = run('manage.py hello -n joe', lambda: manager.run())
+        code = run('manage.py hello -n joe', manager.run)
         out, err = capsys.readouterr()
         assert 'hello joe' in out
 
-        code = run('manage.py hello -h', lambda: manager.run())
+        code = run('manage.py hello -h', manager.run)
         out, err = capsys.readouterr()
         assert 'Prints your name' in out
 
-        code = run('manage.py hello --help', lambda: manager.run())
+        code = run('manage.py hello --help', manager.run)
         out, err = capsys.readouterr()
         assert 'Prints your name' in out
 
@@ -228,19 +321,19 @@ class TestManager:
 
         assert 'verify' in manager._commands
 
-        code = run('manage.py verify --verified', lambda: manager.run())
+        code = run('manage.py verify --verified', manager.run)
         out, err = capsys.readouterr()
         assert 'YES' in out
 
-        code = run('manage.py verify -v', lambda: manager.run())
+        code = run('manage.py verify -v', manager.run)
         out, err = capsys.readouterr()
         assert 'YES' in out
 
-        code = run('manage.py verify', lambda: manager.run())
+        code = run('manage.py verify', manager.run)
         out, err = capsys.readouterr()
         assert 'NO' in out
 
-        code = run('manage.py verify -h', lambda: manager.run())
+        code = run('manage.py verify -h', manager.run)
         out, err = capsys.readouterr()
         assert 'Checks if verified' in out
 
@@ -259,11 +352,11 @@ class TestManager:
 
         assert 'hello' in manager._commands
 
-        code = run('manage.py hello joe', lambda: manager.run())
+        code = run('manage.py hello joe', manager.run)
         out, err = capsys.readouterr()
         assert 'hello joe' in out
 
-        code = run('manage.py hello joe --url=reddit.com', lambda: manager.run())
+        code = run('manage.py hello joe --url=reddit.com', manager.run)
         out, err = capsys.readouterr()
         assert 'hello joe from reddit.com' in out
 
@@ -277,11 +370,11 @@ class TestManager:
 
         assert 'hello' in manager._commands
 
-        code = run('manage.py hello --name=joe', lambda: manager.run())
+        code = run('manage.py hello --name=joe', manager.run)
         out, err = capsys.readouterr()
         assert 'hello joe' in out
 
-        code = run('manage.py hello -h', lambda: manager.run())
+        code = run('manage.py hello -h', manager.run)
         out, err = capsys.readouterr()
         assert 'Your name' in out
 
@@ -295,11 +388,11 @@ class TestManager:
 
         assert 'hello_again' in manager._commands
 
-        code = run('manage.py hello_again --name=joe', lambda: manager.run())
+        code = run('manage.py hello_again --name=joe', manager.run)
         out, err = capsys.readouterr()
         assert 'hello joe' in out
 
-        code = run('manage.py hello_again --name=joe --url=reddit.com', lambda: manager.run())
+        code = run('manage.py hello_again --name=joe --url=reddit.com', manager.run)
         out, err = capsys.readouterr()
         assert 'hello joe from reddit.com' in out
 
@@ -311,12 +404,12 @@ class TestManager:
 
         assert isinstance(manager._commands['simple'], SimpleCommand)
 
-        code = run('manage.py -c Development simple', lambda: manager.run())
+        code = run('manage.py -c Development simple', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'OK' in out
 
-        code = run('manage.py simple -c Development', lambda: manager.run())
+        code = run('manage.py simple -c Development', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'OK' in out
@@ -333,32 +426,13 @@ class TestManager:
 
         assert isinstance(manager._commands['simple'], SimpleCommand)
 
-        code = run('manage.py simple', lambda: manager.run())
+        code = run('manage.py simple', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'Empty' not in out  # config_name is overwritten by default option value
         assert 'Development' in out
         assert 'OK' in out
 
-        code = run('manage.py -c Before simple', lambda: manager.run())
-        out, err = capsys.readouterr()
-        assert code == 0
-        assert 'Before' in out
-        assert 'OK' in out
-
-        code = run('manage.py simple -c After', lambda: manager.run())
-        out, err = capsys.readouterr()
-        assert code == 0
-        assert 'After' in out
-        assert 'OK' in out
-
-        code = run('manage.py -c DoNotShow simple -c NewValue', lambda: manager.run())
-        out, err = capsys.readouterr()
-        assert code == 0
-        assert 'DoNotShow' not in out  # first parameter is ignored
-        assert 'NewValue' in out       # second on is printed
-        assert 'OK' in out
-
     def test_get_usage(self):
 
         manager = Manager(self.app)
@@ -380,14 +454,14 @@ class TestManager:
 
         manager = Manager(self.app)
         manager.add_command('simple', SimpleCommand())
-        code = run('manage.py simple', lambda: manager.run())
+        code = run('manage.py simple', manager.run)
         out, err = capsys.readouterr()
         assert 'OK' in out
 
     def test_run_non_existant_command(self, capsys):
 
         manager = Manager(self.app)
-        run('manage.py simple', lambda: manager.run())
+        run('manage.py simple', manager.run)
         out, err = capsys.readouterr()
         assert 'invalid choice' in err
 
@@ -396,7 +470,7 @@ class TestManager:
         manager = Manager(self.app)
         manager.add_command('simple', SimpleCommand())
 
-        code = run('manage.py simple', lambda: manager.run())
+        code = run('manage.py simple', manager.run)
         out, err = capsys.readouterr()
         assert 0 == code
         assert 'OK' in out
@@ -414,24 +488,27 @@ class TestManager:
 
         manager = Manager(self.app)
 
-        code = run('manage.py simple', lambda: manager.run())
+        code = run('manage.py simple', manager.run)
         out, err = capsys.readouterr()
         assert code == 2
         assert 'OK' not in out
 
-    def test_run_no_name(self):
+    def test_run_no_name(self, capsys):
 
         manager = Manager(self.app)
+        manager.add_command('simple', SimpleCommand())
 
-        code = run('manage.py', lambda: manager.run())
+        code = run('manage.py', manager.run)
+        out, err = capsys.readouterr()
         assert code == 2
+        assert 'simple command' in out
 
     def test_run_good_options(self, capsys):
 
         manager = Manager(self.app)
         manager.add_command('simple', CommandWithOptions())
 
-        code = run('manage.py simple --name=Joe', lambda: manager.run())
+        code = run('manage.py simple --name=Joe', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'Joe' in out
@@ -441,7 +518,7 @@ class TestManager:
         manager = Manager(self.app)
         manager.add_command('simple', CommandWithDynamicOptions('Fred'))
 
-        code = run('manage.py simple', lambda: manager.run())
+        code = run('manage.py simple', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'Fred' in out
@@ -450,7 +527,7 @@ class TestManager:
         manager = Manager(self.app)
         manager.add_command('catch', CommandWithCatchAll())
 
-        code = run('manage.py catch pos1 --foo pos2 --bar', lambda: manager.run())
+        code = run('manage.py catch pos1 --foo pos2 --bar', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert "['pos1', 'pos2', '--bar']" in out
@@ -459,7 +536,7 @@ class TestManager:
         manager = Manager(self.app)
         manager.add_command('simple', CommandWithOptions())
 
-        code = run('manage.py simple --foo=bar', lambda: manager.run())
+        code = run('manage.py simple --foo=bar', manager.run)
         assert code == 2
 
     def test_init_with_flask_instance(self):
@@ -479,7 +556,7 @@ class TestManager:
             raise IndexError()
 
         with raises(IndexError):
-            run('manage.py error', lambda: manager.run())
+            run('manage.py error', manager.run)
 
     def test_run_with_default_command(self, capsys):
         manager = Manager(self.app)
@@ -504,7 +581,7 @@ class TestManager:
                 return 'john'
 
         with hello_john:
-            code = run('manage.py hello', lambda: manager.run())
+            code = run('manage.py hello', manager.run)
             out, err = capsys.readouterr()
             assert 'hello: john' in out
 
@@ -522,7 +599,7 @@ class TestManager:
                 return '\n'  # just hit enter
 
         with hello:
-            code = run('manage.py hello', lambda: manager.run())
+            code = run('manage.py hello', manager.run)
             out, err = capsys.readouterr()
             assert 'hello [romeo]: romeo' in out
 
@@ -532,7 +609,7 @@ class TestManager:
                 return 'juliette'
 
         with hello_juliette:
-            code = run('manage.py hello', lambda: manager.run())
+            code = run('manage.py hello', manager.run)
             out, err = capsys.readouterr()
             assert 'hello [romeo]: juliette' in out
 
@@ -562,29 +639,26 @@ class TestManager:
                 return 'n'
 
         with correct_default:
-            code = run('manage.py hello', lambda: manager.run())
+            code = run('manage.py hello', manager.run)
             out, err = capsys.readouterr()
             assert 'correct [y]: yes' in out
 
         with correct_y:
-            code = run('manage.py hello', lambda: manager.run())
+            code = run('manage.py hello', manager.run)
             out, err = capsys.readouterr()
             assert 'correct [y]: yes' in out
 
         with correct_n:
-            code = run('manage.py hello', lambda: manager.run())
+            code = run('manage.py hello', manager.run)
             out, err = capsys.readouterr()
             assert 'correct [y]: no' in out
 
 
 class TestSubManager:
 
-    TESTING = True
-
     def setup(self):
 
-        self.app = Flask(__name__)
-        self.app.config.from_object(self)
+        self.app = TestApp()
 
     def test_add_submanager(self):
 
@@ -605,7 +679,7 @@ class TestSubManager:
         manager = Manager(self.app)
         manager.add_command('sub_manager', sub_manager)
 
-        code = run('manage.py sub_manager simple', lambda: manager.run())
+        code = run('manage.py sub_manager simple', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'OK' in out
@@ -619,12 +693,12 @@ class TestSubManager:
         manager.add_command('sub_manager', sub_manager)
         manager.add_option('-c', '--config', dest='config', required=False)
 
-        code = run('manage.py sub_manager simple', lambda: manager.run())
+        code = run('manage.py sub_manager simple', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'OK' in out
 
-        code = run('manage.py -c Development sub_manager simple', lambda: manager.run())
+        code = run('manage.py -c Development sub_manager simple', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'OK' in out
@@ -636,27 +710,48 @@ class TestSubManager:
         manager = Manager(self.app)
         manager.add_command('sub_manager', sub_manager)
 
-        code = run('manage.py -h', lambda: manager.run())
+        code = run('manage.py -h', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
         assert 'Example sub-manager' in out
 
-    def test_submanager_usage(self, capsys):
+    def test_submanager_usage_and_help_and_description(self, capsys):
 
-        sub_manager = Manager(usage='Example sub-manager')
+        sub_manager = Manager(usage='sub_manager [--foo]',
+                              help='shorter desc for submanager',
+                              description='longer desc for submanager')
         sub_manager.add_command('simple', SimpleCommand())
 
         manager = Manager(self.app)
         manager.add_command('sub_manager', sub_manager)
 
-        code = run('manage.py sub_manager', lambda: manager.run())
-        out, err = capsys.readouterr()
-        assert code == 2
-        assert 'too few arguments' in err
-
-        code = run('manage.py sub_manager -h', lambda: manager.run())
+        code = run('manage.py -h', manager.run)
         out, err = capsys.readouterr()
         assert code == 0
+        assert 'sub_manager [--foo]' not in out
+        assert 'shorter desc for submanager' in out
+        assert 'longer desc for submanager' not in out
+
+        code = run('manage.py sub_manager', manager.run)
+        out, err = capsys.readouterr()
+        assert code == 2
+        assert 'sub_manager [--foo]' in out
+        assert 'shorter desc for submanager' not in out
+        assert 'longer desc for submanager' in out
+        assert 'simple command' in out
+
+        code = run('manage.py sub_manager -h', manager.run)
+        out, err = capsys.readouterr()
+        assert code == 0
+        assert 'sub_manager [--foo]' in out
+        assert 'shorter desc for submanager' not in out
+        assert 'longer desc for submanager' in out
+        assert 'simple command' in out
+
+        code = run('manage.py sub_manager simple -h', manager.run)
+        out, err = capsys.readouterr()
+        assert code == 0
+        assert 'sub_manager [--foo] simple [-h]' in out
         assert 'simple command' in out
 
     def test_submanager_has_no_default_commands(self):
